@@ -88,6 +88,7 @@
 #include <mail_queue.h>
 #include <mail_conf.h>
 #include <mail_params.h>
+#include <mail_version.h>
 #include <mail_proto.h>
 
 /* Application-specific. */
@@ -113,6 +114,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
     const char *error_text;
     char   *attr_name;
     char   *attr_value;
+    int     rec_flags = (msg_verbose ? REC_FLAG_NONE : REC_FLAG_DEFAULT);
 
 #define TEXT_RECORD(rec_type) \
 	    (rec_type == REC_TYPE_CONT || rec_type == REC_TYPE_NORM)
@@ -131,10 +133,10 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
     /*
      * Now look at the rest.
      */
-    for (;;) {
+    do {
 	if (flags & PC_FLAG_OFFSET)
 	    offset = vstream_ftell(fp);
-	rec_type = rec_get(fp, buffer, 0);
+	rec_type = rec_get_raw(fp, buffer, 0, rec_flags);
 	if (rec_type == REC_TYPE_ERROR)
 	    msg_fatal("record read error");
 	if (rec_type == REC_TYPE_EOF)
@@ -159,6 +161,13 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    vstream_printf("%s: %s", rec_type_name(rec_type),
 			   asctime(localtime(&time)));
 	    break;
+	case REC_TYPE_PTR:			/* pointer */
+	    vstream_printf("%s: ", rec_type_name(rec_type));
+	    vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
+	    VSTREAM_PUTCHAR('\n');
+	    if (rec_goto(fp, STR(buffer)) == REC_TYPE_ERROR)
+		msg_fatal("bad pointer record, or input is not seekable");
+	    break;
 	case REC_TYPE_CONT:			/* REC_TYPE_FILT collision */
 	    if (!in_message)
 		vstream_printf("%s: ", rec_type_name(rec_type));
@@ -176,6 +185,13 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
 	    VSTREAM_PUTCHAR('\n');
 	    break;
+	case REC_TYPE_DTXT:
+	    if (msg_verbose) {
+		vstream_printf("%s: ", rec_type_name(rec_type));
+		vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
+		VSTREAM_PUTCHAR('\n');
+	    }
+	    break;
 	case REC_TYPE_MESG:
 	    vstream_printf("*** MESSAGE CONTENTS %s ***\n", VSTREAM_PATH(fp));
 	    in_message = 1;
@@ -200,7 +216,9 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 			       asctime(localtime(&time)));
 		break;
 	    }
-	    /* FALLTHROUGH */
+	    vstream_printf("%s: %s=%s\n", rec_type_name(rec_type),
+			   attr_name, attr_value);
+	    break;
 	default:
 	    vstream_printf("%s: %s\n", rec_type_name(rec_type), STR(buffer));
 	    break;
@@ -211,7 +229,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	 * In case the next record is broken.
 	 */
 	vstream_fflush(VSTREAM_OUT);
-    }
+    } while (rec_type != REC_TYPE_END);
 }
 
 /* usage - explain and terminate */
@@ -222,6 +240,8 @@ static NORETURN usage(char *myname)
 	      myname);
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 int     main(int argc, char **argv)
 {
     VSTRING *buffer;
@@ -241,6 +261,11 @@ int     main(int argc, char **argv)
     char  **cpp;
     int     tries;
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     /*
      * To minimize confusion, make sure that the standard file descriptors
      * are open before opening anything else. XXX Work around for 44BSD where
